@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-#my $Id = '$Id: countub.pl 535 2010-01-11 02:42:50Z pro $';
+#my $Id = '$Id: countub.pl 594 2010-01-30 23:10:17Z pro $';
 
 =copyright
 counting users-bytes from dchub for mrtg or cacti (snmpd)
@@ -11,12 +11,13 @@ use Net::DirectConnect;
 print("usage: $0 [adc|dchub://]host[:port] [bot_nick] [share_delim]\n"), exit if !$ARGV[0];
 $ARGV[0] =~ m|^(?:\w+\://)?(.+?)(?:\:(\d+))?$|;
 my $dc = Net::DirectConnect->new(
-  'host' => $ARGV[0],
-  'Nick' => ( $ARGV[1] or 'dcpppCnt' ),    #'log' => sub { },    # no logging
+  'host'       => $ARGV[0],
+  'Nick'       => ( $ARGV[1] or 'dcpppCnt' ),    #'log' => sub { },    # no logging
+  auto_GetINFO => 1,
 );
 my ($share) = 0;
-$dc->wait_connect();                       #for 1 .. 3;
-$dc->cmd('GetINFO') if $dc->{nmdc};
+$dc->wait_connect();                             #for 1 .. 3;
+#$dc->cmd('GetINFO') if $dc->{nmdc};
 $dc->work(5);
 if   ( $dc->{nmdc} ) { $share += $dc->{'NickList'}{$_}{'sharesize'} for keys %{ $dc->{'NickList'} }; }
 else                 { $share += $dc->{'peers'}{$_}{INF}{'SS'}      for keys %{ $dc->{'peers'} }; }
